@@ -1,5 +1,7 @@
 package Term::UI;
 
+use if $] > 5.017, 'deprecate';
+
 use Carp;
 use Params::Check qw[check allow];
 use Term::ReadLine;
@@ -11,7 +13,7 @@ use strict;
 BEGIN {
     use vars        qw[$VERSION $AUTOREPLY $VERBOSE $INVALID];
     $VERBOSE    =   1;
-    $VERSION    =   '0.28';
+    $VERSION    =   '0.40';
     $INVALID    =   loc('Invalid selection, please try again: ');
 }
 
@@ -101,7 +103,7 @@ You can indicate that the user is allowed to enter multiple answers by
 toggling the C<multi> flag. Note that a list of answers will then be
 returned to you, rather than a simple string.
 
-By specifying an C<allow> hander, you can yourself validate the answer
+By specifying an C<allow> handler, you can yourself validate the answer
 a user gives. This can be any of the types that the Params::Check C<allow>
 function allows, so please refer to that manpage for details.
 
@@ -120,7 +122,7 @@ sub get_reply {
     my %hash = @_;
 
     my $tmpl = {
-        default     => { default => undef,  strict_type => 1 },
+        default     => { default => undef,  strict_type => 0 },
         prompt      => { default => '',     strict_type => 1, required => 1 },
         choices     => { default => [],     strict_type => 1 },
         multi       => { default => 0,      allow => [0, 1] },
@@ -131,6 +133,10 @@ sub get_reply {
     my $args = check( $tmpl, \%hash, $VERBOSE )
                 or ( carp( loc(q[Could not parse arguments]) ), return );
 
+    # Check for legacy default on multi=1
+    if ($args->{multi} and ref($args->{default}) ne "ARRAY") {
+        $args->{default} = [ $args->{default} ];
+    }
 
     ### add this to the prompt to indicate the default
     ### answer to the question if there is one.
@@ -147,12 +153,21 @@ sub get_reply {
 
             ### so this choice is the default? add it to 'prompt_add'
             ### so we can construct a "foo? [DIGIT]" type prompt
-            $prompt_add = $i if (defined $args->{default} and $choice eq $args->{default});
+            if (defined $args->{default}) {
+                if ($args->{multi}) {
+                    push @$prompt_add, $i if (scalar(grep { m/^$choice$/ } @{$args->{default}}));
+                }
+                else {
+                    $prompt_add = $i if ($choice eq $args->{default});
+                }
+            }
 
             ### create a "DIGIT> choice" type line
             $args->{print_me} .= sprintf "\n%3s> %-s", $i, $choice;
         }
 
+        $prompt_add = join(" ", @$prompt_add) if ($args->{multi});
+
         ### we listed some choices -- add another newline for
         ### pretty printing
         $args->{print_me} .= "\n" if $i;
@@ -163,7 +178,12 @@ sub get_reply {
     ### no choices, but a default? set 'prompt_add' to the default
     ### to construct a 'foo? [DEFAULT]' type prompt
     } elsif ( defined $args->{default} ) {
-        $prompt_add = $args->{default};
+        if ($args->{multi} and ref($args->{default}) eq "ARRAY") {
+            $prompt_add = join(" ", @{$args->{default}});
+        }
+        else {
+            $prompt_add = $args->{default};
+        }
     }
 
     ### we set up the defaults, prompts etc, dispatch to the readline call
@@ -247,7 +267,7 @@ sub _tt_readline {
 
     my ($default, $prompt, $choices, $multi, $allow, $prompt_add, $print_me);
     my $tmpl = {
-        default     => { default => undef,  strict_type => 1,
+        default     => { default => undef,  strict_type => 0,
                             store => \$default },
         prompt      => { default => '',     strict_type => 1, required => 1,
                             store => \$prompt },
@@ -255,7 +275,7 @@ sub _tt_readline {
                             store => \$choices },
         multi       => { default => 0,      allow => [0, 1], store => \$multi },
         allow       => { default => qr/.*/, store => \$allow, },
-        prompt_add  => { default => '',     store => \$prompt_add },
+        prompt_add  => { default => '',     store => \$prompt_add, strict_type => 1 },
         print_me    => { default => '',     store => \$print_me },
     };
 
@@ -266,27 +286,42 @@ sub _tt_readline {
     history( $print_me ) if $print_me;
 
 
-    ### we might have to add a default value to the prompt, to
-    ### show the user what will be picked by default:
-    $prompt .= " [$prompt_add]: " if $prompt_add;
+    if ($prompt_add) {
+        ### we might have to add a default value to the prompt, to
+        ### show the user what will be picked by default:
+        $prompt .= " [$prompt_add]: " ;
+    }
+    else {
+        $prompt .= " : ";
+    }
 
 
     ### are we in autoreply mode?
     if ($AUTOREPLY) {
 
-        ### you used autoreply, but didnt provide a default!
+        ### you used autoreply, but didn't provide a default!
         carp loc(
             q[You have '%1' set to true, but did not provide a default!],
             '$AUTOREPLY'
         ) if( !defined $default && $VERBOSE);
 
         ### print it out for visual feedback
-        history( join ' ', grep { defined } $prompt, $default );
+        if ($multi and defined($default)) {
+            history( join ' ', grep { defined } $prompt, @$default );
+            ### and return the default
+            return @$default;
+        }
+        else {
+            history( join ' ', grep { defined } $prompt, $default );
+            ### and return the default
+            return $default;
+        }
 
-        ### and return the default
-        return $default;
     }
 
+    if ($multi and defined($default)) {
+        $default = join(' ', @$default);
+    }
 
     ### so, no AUTOREPLY, let's see what the user will answer
     LOOP: {
@@ -330,15 +365,14 @@ sub _tt_readline {
                     ### the choices, because humans want to start counting
                     ### at 1, not at 0
                     push @rv, $choices->[ $answer - 1 ]
-                        if $answer > 0 && defined $choices->[ $answer - 1];
+                        if $answer > 0 && defined $choices->[ $answer - 1 ];
                 }
             }
 
         ### no fixed list of choices.. just check if the answers
         ### (or otherwise the default!) pass the allow handler
         } else {
-            push @rv, grep { allow( $_, $allow ) }
-                        scalar @answers ? @answers : ($default);
+            push @rv, grep { allow( $_, $allow ) } @answers;
         }
 
         ### if not all the answers made it to the return value list,
